
--
-- 9 94 1.9 src/com/tivoli/pd/jaudit/AMaudit_schema51.sql, pd.jaudit, am510, 031010a 9/30/03 16:28:20
--
-- Component Name: pd.jaudit
-- File Name: AMaudit_schema51.sql
--
-- IBM Confidential
-- Object Code Only Source Materials
-- 5747-SM3
-- (c) Copyright International Business Machines Corp. 2003
-- The source code for this program is not published or otherwise divested
-- of its trade secrets, irrespective of what has been deposited with the
-- U.S. Copyright Office.
--
-- -----------------------------------------------------------------------
--
--  Description:
--
--  Audit Database table definitions. Assumes that database has already been
--  created and a current connection exists. This allows flexibility in
--  how the database is actually created by separating table creation into
--  this script.
--
--

-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Supported Languages:
-- ====================
-- us - US English


-- Archive control table
--   name is the achive log name
-- echo "Creating table archives";
drop table AMAudit.archives;
create table AMAudit.archives
(
    name                   CHAR(250) FOR bit data not NULL PRIMARY KEY,
    start_time             timestamp not NULL, 
    end_time               timestamp not NULL,
    start_sn               integer not NULL,
    end_sn                 integer not NULL,
    tblname                Char(32),   
    isloaded               Smallint not NULL,
    audit_sig              long varchar FOR bit data not NULL, 
    integrity              CHAR(64)  FOR bit data not NULL
);
--    sig_key_id             Smallint not NULL REFERENCES keys(key_id));



CREATE UNIQUE INDEX AMAudit.archives ON AMAudit.archives(name);
--
-- This table contains information that applies to the entire audit
-- system. It contains only one row
-- 
-- echo "Creating table system";
drop table AMAudit.system;
create table AMAudit.system
(
   first_sn                  integer not NULL,
   next_sn                   integer not NULL
--   first_elemnum             integer not NULL,
--   next_elemnum              integer not NULL
);
--   audit_int                 VARCHAR(64)  FOR bit data not NULL,  
--   archive_int               VARCHAR(64)  FOR bit data not NULL ,
--   events_int                VARCHAR(64)  FOR bit data not NULL ,
--   sources_int               VARCHAR(64)  FOR bit data not NULL ,
--   auth_ent_int              VARCHAR(64)  FOR bit data not NULL ,
--   auth_role_int             VARCHAR(64)  FOR bit data not NULL ,
--   afctd_ent_type_int        VARCHAR(64)  FOR bit data not NULL ,
--   keys_int                  VARCHAR(64)  FOR bit data not NULL ,
--   event_sevs_int            VARCHAR(64)  FOR bit data not NULL ,
--   comp_types_int            VARCHAR(64)  FOR bit data not NULL ,
--   sig_key_id                Smallint not NULL REFERENCES keys(key_id),
--   system_int                VARCHAR(64)  FOR bit data not NULL
--);


-- Note this insert statement here is for a case where,
-- When the AMAuditServer is still running,
-- and the AMaudit_create_tables.bat has been called,
-- (which drops all the tables and creates new tables,which causes the
-- AMAudit.system table to be empty.)
-- The AMAuditServer does not update the AMAudit.system table unless it has
-- some values for first_sn and next_sn. So we are inserting values
-- into the AMAudit.system table.

insert into AMAudit.system(first_sn, next_sn) values(1, 1);


--CREATE INDEX AMAudit.system ON AMAudit.system(system_int);

drop table AMAudit.auditEvents;
create table AMAudit.auditEvents
(
   serialNum            integer not NULL PRIMARY KEY,
   eventType            Varchar(254),
   version              integer not NULL,
   srcDateTime          timestamp,
   createDateTime       timestamp
);     

drop table AMAudit.elements;
create table AMAudit.elements
(
   name                 Varchar(254),
   value                Varchar(1024),
   elementIndex         integer not NULL,
   parent               integer not NULL,
   serialNum            integer not NULL
);

drop table AMAudit.attributes;
create table AMAudit.attributes
(
   name                 Varchar(254),
   value                Varchar(1024),
   elementIndex         integer not NULL,
   serialNum            integer not NULL
);



drop view AMAudit.auditrecord_test;


create view AMAudit.auditrecord_test as
select cast(AMAudit.auditEvents.serialNum as char(4)) sn,
       cast(AMAudit.auditEvents.eventType as char(12)) event,
       cast(AMAudit.elements.elementIndex as char(4)) ind, 
       cast(AMAudit.elements.name as char(12)) elemname,
       cast(AMAudit.elements.value as char(12)) elemval, 
       cast(parent as char(4)) par,
       cast(AMAudit.attributes.name as char(12)) attrname,
       cast(AMAudit.attributes.value as char(12)) attrval
       from AMAudit.elements LEFT OUTER JOIN AMAudit.attributes 
       ON AMAudit.elements.elementIndex = AMAudit.attributes.elementIndex
       AND AMAudit.elements.serialNum = AMAudit.attributes.serialNum, AMAudit.auditEvents 
       WHERE AMAudit.auditEvents.serialNum = AMAudit.elements.serialNum;



drop view AMAudit.elements_t;

create view AMAudit.elements_t as
select cast(name as char(12)) name,
       cast(value as char(12)) value,
       elementIndex index,
       parent parent,
       serialNum sn
  from AMAudit.elements ; 


drop view AMAudit.auditevents_t;

create view AMAudit.auditevents_t as
select cast(eventType as char(12)) event,
       SUBSTR(CHAR(srcDateTime),1,16) sourcetime, 
       SUBSTR(CHAR(createDateTime),1,16) createtime, 
       serialNum 
  from AMAudit.auditEvents ; 


drop view AMAudit.attributes_t;

create view AMAudit.attributes_t as
select cast(name as char(12)) name,
       cast(value as char(12)) value,
       elementIndex index,
       serialNum 
  from AMAudit.attributes ; 


drop view AMAudit.target_t;
create view AMAudit.target_t as
select DISTINCT cast(a.eventType as char(12)) event,
       cast(a.serialNum as char(4)) sn,
       SUBSTR(CHAR(createDateTime),1,16) createtime,
       cast(e.value as char(80)) target
       from  AMAudit.auditEvents a,
       AMAudit.elements e where a.serialNum = e.serialNum AND
       e.name='target';

drop view AMAudit.principal_t;
create view AMAudit.principal_t as
select cast(a.serialNum as char(4)) sn,
       SUBSTR(CHAR(createDateTime),1,16) createtime,
       cast(e.value as char(12)) principal
       from AMAudit.auditEvents a,
       AMAudit.elements e where a.serialNum = e.serialNum AND
       e.name='principal';

drop view AMAudit.result_t;
create view AMAudit.result_t as
select cast(a.serialNum as char(4)) sn,
       SUBSTR(CHAR(createDateTime),1,16) createtime,
       cast(e.value as char(12)) result
       from AMAudit.auditEvents a,
       AMAudit.elements e where a.serialNum = e.serialNum AND
       e.name='result';

drop view AMAudit.authorization_t;
create view AMAudit.authorization_t as
select cast(a.sn as char(4)) sn,
       cast(a.event as char(12)) event,
       cast(a.createtime as char(16)) sourcetime,
       cast(b.principal as char(16)) principal,
       cast(c.result as char(16)) result,
       cast(a.target as char(80)) target
  from AMAudit.target_t a, AMAudit.principal_t b, AMAudit.result_t c where a.sn=b.sn AND b.sn=c.sn AND a.createtime=b.createtime AND a.createtime=c.createtime ;

