#ifndef lint
static char sccsid[]="@(#)89 1.8 src/ivmgrd/pdadminapi/pdadminapi_demo/pdadminapi_demo.c, pd.admin, am510, 031010a 9/11/02 09:01:37";
#endif
/*
 * Component Name: ivadmin adk
 * File Name: pdadminapi_demo.c
 *
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2001, 2002
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplicaion or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ivadminapi.h"

/*
 *
 * This program demonstrates the use of the ivadminapi for various
 * management tasks.
 *
 */

/*
 * Global constants
 * These should be changed to match your environment before compiling and
 * running the program.
 */
char *sec_masterpwd = "password";     /* the password for the sec_master user */
char *ldap_suffix = "o=tivoli,c=us";  /* the LDAP suffix for Policy Director users */

/*****************************************************************************
 * Prints the response text from an ivadmin API command.
 * Frees the response.
 *****************************************************************************/

void processResponse(ivadmin_response rsp, char *api)
{
   unsigned long i=0;

   if (ivadmin_response_getok(rsp)) {
      fprintf(stderr,"   %s : succeeded\n", api);
   } else {
      fprintf(stderr,"   %s : failed\n", api);
   }

   for (i=0; i<ivadmin_response_getcount(rsp); i++) {
      fprintf(stderr,"   %s : %s\n", api, ivadmin_response_getmessage(rsp, i));
   }

   ivadmin_free(rsp);
}

int main(int argc, char ** argv)
{
    int success = 1;
    unsigned long status;
    ivadmin_context ctx;
    ivadmin_response rsp;
    const char *username = "testuser";
    char *userdn;
    char *groupname = "testgroup";
    char *groupdn;

    /*
     * Before doing and management operations it is necessary to create
     * a context for communication with the Access Manager policy server.
     */
    printf("Creating context for communication with the Access Manager policy server\n");
    status = ivadmin_context_createdefault2("sec_master", sec_masterpwd, 
                                             IVADMIN_DOMAIN_MANAGEMENT,
                                             &ctx, &rsp);
    processResponse(rsp, "ivadmin_context_createdefault2");
    if (status != IVADMIN_TRUE) {
        return 0; /* The context create call failed so we should just exit */
    }

    /*
     * Create a Policy Director User
     */
    if ((userdn =
         (char *)malloc(strlen(ldap_suffix) + strlen(username) + 5)) == NULL)
    {
        fprintf (stderr, "Error allocating memory for userdn.\n");
        return 0;
    }
    sprintf(userdn, "cn=%s,%s", username, ldap_suffix);
    printf("Creating Policy Director user: %s\n", username);
    status = ivadmin_user_create3(ctx, username, userdn,
                                  username, username, "ChangeMe1", 0, NULL,
                                  IVADMIN_FALSE, IVADMIN_FALSE, &rsp);
    processResponse(rsp, "ivadmin_user_create3");
    free(userdn);

    /*
     * Set the account to valid for the new user
     */
    printf("Setting the account to valid for user: %s\n", username);
    status = ivadmin_user_setaccountvalid(ctx, username, IVADMIN_TRUE, &rsp);
    processResponse(rsp, "ivadmin_user_setaccountvalid");

    /*
     * Change the password of the new user
     */
    printf("Changing the password for user: %s\n", username);
    status = ivadmin_user_setpassword(ctx, username, "ChangeMe2", &rsp);
    processResponse(rsp, "ivadmin_user_setpassword");

    /*
     * Create a new group
     */
    
    if ((groupdn =
         (char *)malloc(strlen(ldap_suffix) + strlen(groupname) + 5)) == NULL)
    {
        fprintf (stderr, "Error allocating memory for groupdn.\n");
        return 0;
    }
    sprintf(groupdn, "cn=%s,%s", groupname, ldap_suffix);
    printf("Creating a new group: %s\n", groupname);
    status = ivadmin_group_create2(ctx, groupname, groupdn, groupname, NULL, &rsp);
    processResponse(rsp, "ivadmin_group_create2");
    free(groupdn);

    /*
     * Add the new user to the new group
     */
    printf("Adding user: %s to group: %s\n", username, groupname);

    status = ivadmin_group_addmembers(ctx, groupname, 1, &username, &rsp);
    processResponse(rsp, "ivadmin_group_addmembers");

    /*
     * Delete the group
     */
    printf("Deleting group: %s\n", groupname);
    status = ivadmin_group_delete2(ctx, groupname, IVADMIN_TRUE, &rsp);
    processResponse(rsp, "ivadmin_group_delete2");

    /*
     * Delete the user
     */
    printf("Deleting user: %s\n", username);
    status = ivadmin_user_delete2(ctx, username, IVADMIN_TRUE, &rsp);
    processResponse(rsp, "ivadmin_user_delete2");

    /*
     * Delete the context
     */
    printf("Deleting the context to the Access Manager policy server\n");
    status = ivadmin_context_delete(ctx, &rsp);
    processResponse(rsp, "ivadmin_context_delete");

    ivadmin_free(ctx);

    return 1;
}
