/*
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2001
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplicaion or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Copyright (C) Tivoli Systems/IBM.  2000
 *
 * NAME
 *     pdaudit - Policy Director Auditing Service header
 *
 * DESCRIPTION
 *     This header file contains the definitions of Auditing Service
 *     specific macros and the "C" API functions prototypes.
 */

#ifndef AM_AUDITTYPES_H
#define AM_AUDITTYPES_H

#include <IVTypes.h>

/* ************************************************************************ */
/* DEPRECATED:                                                              */
/* ----------                                                               */
/* This is for PDAuditEventAddCommon, which will be deprecated.             */
/* Please start to use PDAuditEventSetNameValue to set these as name/value pairs */
typedef enum PDAuditEnumResource {
    PD_AUDIT_RESOURCE_AZN,
    PD_AUDIT_RESOURCE_PROCESS,
    PD_AUDIT_RESOURCE_TCB,
    PD_AUDIT_RESOURCE_CREDENTIAL,
    PD_AUDIT_RESOURCE_POLICY,
    PD_AUDIT_RESOURCE_GENERAL,
    PD_AUDIT_RESOURCE_APPLICATION,
    PD_AUDIT_RESOURCE_AUTHENTICATION
} PDAuditEnumResource;

typedef enum PDAuditEnumSystem {
    PD_AUDIT_SYSTEM_UNIX,
    PD_AUDIT_SYSTEM_NT
} PDAuditEnumSystem;

typedef enum AMAuditEnumOutcome {
    PD_AUDIT_OUTCOME_SUCCESS,
    PD_AUDIT_OUTCOME_FAILURE,
    PD_AUDIT_OUTCOME_PENDING,
    PD_AUDIT_OUTCOME_UNKNOWN
} AMAuditEnumOutcome;
#define PDAuditEnumOutcome AMAuditEnumOutcome
#ifdef __cplusplus
extern "C" {
#endif
typedef IVTbool (*AMAuditFilterFunction)(const IVTuint32, const AMAuditEnumOutcome);
#define PDAuditFilterFunction AMAuditFilterFunction
#ifdef __cplusplus
}
#endif

/* Override the pdAuditCommon's cred fields using PDAuditEventSetNameValue */
#define PD_AUDIT_COMMON_RESOURCE              "amtrc resource"
#define PD_AUDIT_VALUE_RESOURCE_AZN             "0"
#define PD_AUDIT_VALUE_RESOURCE_PROCESS         "1"
#define PD_AUDIT_VALUE_RESOURCE_TCB             "2"
#define PD_AUDIT_VALUE_RESOURCE_CREDENTIAL      "3"
#define PD_AUDIT_VALUE_RESOURCE_POLICY          "4"
#define PD_AUDIT_VALUE_RESOURCE_GENERAL         "5"
#define PD_AUDIT_VALUE_RESOURCE_APPLICATION     "6"
#define PD_AUDIT_VALUE_RESOURCE_AUTHENTICATION  "7"

#define PD_AUDIT_COMMON_ORIGINATING_LOCATION   "amtrc location"

#define PD_AUDIT_COMMON_LINKAGE                "amtrc link"

#define PD_AUDIT_COMMON_FAIL_STATUS            "amtrc status"

#define PD_AUDIT_COMMON_PROTECTED_OBJECT       "amtrc target"

/* Generic string for a null or not specified field.                                   */
/* Use for unspecified PD_AUDIT_CRED_USER_INFO/MECH_ID/PRINCIPAL_NAME/PRINCIPAL_DOMAIN */
/* #define PD_AUDIT_VALUE_NOTSPECIFIED            "Not specified"                      */

/* Process Information, will be added to audit record if any of these are set... */
#define PD_AUDIT_COMMON_SYSTEM                 "amtrc architecture"
#define PD_AUDIT_VALUE_SYSTEM_UNIX              "0"
#define PD_AUDIT_VALUE_SYSTEM_WINDOWS           "1"

#define PD_AUDIT_COMMON_PID                    "amtrc pid"
#define PD_AUDIT_COMMON_UID                    "amtrc uid"
#define PD_AUDIT_COMMON_EID                    "amtrc eid"
#define PD_AUDIT_COMMON_GID                    "amtrc gid"
#define PD_AUDIT_COMMON_EGID                   "amtrc egid"
/* End of Process Information */

/* AZN Information, will be added to audit record if any of these are set... */
#define PD_AUDIT_COMMON_AZN_PERMISSION         "amtrc permission"
#define PD_AUDIT_COMMON_AZN_RESULT             "amtrc result"
#define PD_AUDIT_COMMON_AZN_RESULT_QUALIFIER   "amtrc qualifier"
/* End of AZN Information */

/* Audited Attributes from a user's credential */
#define PD_AUDIT_CRED_USER_INFO         "amtrc accessor name"
#define PD_AUDIT_CRED_MECH_ID           "amtrc principal auth"
#define PD_AUDIT_CRED_PRINCIPAL_NAME    "amtrc principal"
#define PD_AUDIT_CRED_PRINCIPAL_DOMAIN  "amtrc principal domain"

/* special case values for credential attributes */

#define PD_AUDIT_VALUE_MECH_ID_NULL   "Not specified"
/* If the credential's mech id is 0, then set the audit mech id using
    PDAuditEventSetNameValue(event_h,
                             PD_AUDIT_CRED_MECH_ID,
                             PD_AUDIT_VALUE_MECH_ID_NULL,
                             &status)
*/

#define PD_AUDIT_VALUE_PRINCIPAL_DOMAIN_NULL   "Default"
/* If the credential's principal domain is 0, then set the domain using
    PDAuditEventSetNameValue(event_h,
                             PD_AUDIT_CRED_PRINCIPAL_DOMAIN,
                             PD_AUDIT_VALUE_PRINCIPAL_DOMAIN_NULL,
                             &status)
*/


#define PD_AUDIT_VALUE_PRINCIPAL_UNAUTH     "Unauth"
/* If credential's mech id is "unauth", then use
    PDAuditEventSetNameValue(event_h,
                             PD_AUDIT_CRED_PRINCIPAL_NAME,
                             PD_AUDIT_VALUE_PRINCIPAL_UNAUTH,
                             &status)
*/

/*
 * Audit Record Version
 */
#define PD_AUDIT_RECORD_REV     0x102

/*
 Audit interface text character encoding
*/
#define AM_AUDIT_ENCODING_UTF8 "UTF8"
#define PD_AUDIT_ENCODING_UTF8 AM_AUDIT_ENCODING_UTF8
#define AM_AUDIT_ENCODING_LCP  NULL
#define PD_AUDIT_ENCODING_LCP  AM_AUDIT_ENCODING_LCP

#endif
