/*
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2001
 * US Government Users Restricted Rights - Use, duplicaion or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * All Rights Reserved.
 *
 * File Name: azn_deprecated.h
 */

#ifndef _azn_deprecated_h
#define _azn_deprecated_h

/**************************************************************************
*  WARNING!   WARNING!   WARNING!   WARNING!   WARNING!   WARNING!
*
* This file contains all aznAPI interfaces that have been deprecated.
* Please start using the new interfaces that replace the deprecated
* interfaces as soon as possible.
*
**************************************************************************/

#include <ogauthzn.h>

#ifdef __cplusplus
extern "C" {
#endif

/**************************************************************************
*                     Deprecated azn_init_* attributes
*
* The following attributes passed to azn_initialize() have been deprecated:
*
* azn_init_namespace_location
* azn_init_tcp_port
* azn_init_udp_port
* azn_init_qop
* azn_init_remote_ns_loc
* azn_init_max_handle_groups
*
* The following entries set in the aznAPI configuration file (that
* correspond to the above deprecated azn_initialize attributes) have
* been deprecated:
*
* namespace-location
* tcp_port
* udp_port
* remote-qop
* remote-ns-loc
* max-handle-groups
*
**************************************************************************/
/*
 * CDS namespace location for exporting the RPC endpoints for local
 * policy cache updates.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_namespace_location;

/* The TCP port upon which policy cache updates should be received. */
AZN_EXTSPEC_CONST azn_string_t azn_init_tcp_port;

/* The UDP port upon which policy cache updates should be received. */
AZN_EXTSPEC_CONST azn_string_t azn_init_udp_port;

/*
 * Quality of protection for communications with IVAcld.
 * One of "none", "integrity" or "privacy".
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_qop;

/* List of namespace entries for communications with IVAcld. */
AZN_EXTSPEC_CONST azn_string_t azn_init_remote_ns_loc;

/*
 * The max number of handle groups (4k handles in each) permitted.
 * The default is
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_max_handle_groups;

/**************************************************************************
 *                     Deprecated azn_init_master_xxx attributes
 *
 * The following communications attributes
 * info attribute list are being deprecated.
 *
 *   azn_init_master_dn  --  The DN of the policy server
 **************************************************************************/
AZN_EXTSPEC_CONST azn_string_t azn_init_master_dn;

/**************************************************************************
 *
 *                     Deprecated azn_perminfo attributes
 *
 **************************************************************************/
/* 
 * The following attributes that can be received by a caller
 * of the azn_decision_access_allowed_ext() call in the permission
 * info attribute list are being deprecated. They're constant name
 * does not match:
 *
 *  azn_perminfo_al                  Audit level (uint).
 *  azn_perminfo_al_ulong            Audit level (unsigned long).
 *  azn_perminfo_wm                  Warning mode (bool).
 *  azn_perminfo_wm_ulong            Warning mode (unsigned long).
 *  azn_perminfo_wm_permitted        Access permitted by warning mode (bool).
 *  azn_perminfo_wm_permitted_ulong  Access permitted by warning mode
 *                                   (boolean retuned in unsigned long value).
 *  azn_perminfo_al_all
 *  azn_perminfo_al_none
 *  azn_perminfo_al_permit
 *  azn_perminfo_al_deny
 *  azn_perminfo_al_error
 *  azn_perminfo_al_admin
 *
 * These attributes have been replaced by the following new attributes
 * defined in ogauthzn.h
 *
 *  azn_perminfo_auditlevel_ulong
 *  azn_perminfo_warningmode_ulong
 *  azn_perminfo_warningmodepermitted_ulong
 *  azn_perminfo_auditlevel_all
 *  azn_perminfo_auditlevel_none
 *  azn_perminfo_auditlevel_permit
 *  azn_perminfo_auditlevel_deny
 *  azn_perminfo_auditlevel_error
 *  azn_perminfo_auditlevel_admin
 */
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_al;
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_al_ulong;
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_wm;
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_wm_ulong;
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_wm_permitted;
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_wm_permitted_ulong;
#if !defined(AZNDLL)
AZN_EXTSPEC_CONST unsigned int azn_perminfo_al_none;
AZN_EXTSPEC_CONST unsigned int azn_perminfo_al_permit;
AZN_EXTSPEC_CONST unsigned int azn_perminfo_al_deny;
AZN_EXTSPEC_CONST unsigned int azn_perminfo_al_error;
AZN_EXTSPEC_CONST unsigned int azn_perminfo_al_admin;
AZN_EXTSPEC_CONST unsigned int azn_perminfo_al_all;
#endif


/**************************************************************************
 *
 * This uint qop attribute and its values is deprecated
 *
 *  azn_perminfo_qop_uint            Quality of Protection (uint).
 *  azn_perminfo_qop_uint_none
 *  azn_perminfo_qop_uint_integrity
 *  azn_perminfo_qop_uint_privacy
 *
 * and replaced by a ulong equivalent:
 *
 *  azn_perminfo_qop_ulong           Quality of Protection (ulong).
 *  azn_perminfo_qop_value_none
 *  azn_perminfo_qop_value_integrity
 *  azn_perminfo_qop_value_privacy
 *
 **************************************************************************/
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_qop_uint;
AZN_EXTSPEC_CONST unsigned long azn_perminfo_qop_uint_none;
AZN_EXTSPEC_CONST unsigned long azn_perminfo_qop_uint_integrity;
AZN_EXTSPEC_CONST unsigned long azn_perminfo_qop_uint_privacy;


/**************************************************************************
*                     Deprecated azn_cred_* Attributes
*
* The following azn_cred_* attributes have been deprecated.
*
* azn_cred_dce_name
* azn_cred_uraf_name
*
* They are all replaced by the following attribute defined in ogauthzn.h
*
* azn_cred_authzn_id
*
* azn_cred_group_names is replaced by azn_cred_group_registry_ids
*
* azn_cred_ldap_dn is replaced by azn_cred_registry_id
**************************************************************************/

/* the DCE name used to build these authzn credentials */
AZN_EXTSPEC_CONST azn_string_t azn_cred_dce_name;

/* the URAF name used to build these authzn credentials */
AZN_EXTSPEC_CONST azn_string_t azn_cred_uraf_name;

/* the string group name memberships of this entity */
AZN_EXTSPEC_CONST azn_string_t azn_cred_group_names;

/* For LDAP credentials only: contains the LDAP DN of the principal */
AZN_EXTSPEC_CONST azn_string_t azn_cred_ldap_dn;


/**************************************************************************
*                     Deprecated azn_operation_* Attribute
*
* The following azn_operation_* attribute has been deprecated.
*
* azn_operation_bypasstod
*
* It is replaced by the following attribute defined in ogauthzn.h
*
* azn_operation_bypasspop
*
**************************************************************************/
AZN_EXTSPEC_CONST azn_string_t azn_operation_bypasstod;


/**************************************************************************
*    Deprecated PD Protected Object Entitlement Service minor return codes.
*
* The following minor return codes have been deprecated:
*             AZN_ENT_PDPOBJ_INVALID_SVCINFO_HDL
*             AZN_ENT_PDPOBJ_INVALID_ARG_COUNT
*             AZN_ENT_PDPOBJ_INVALID_ARG_ARRAY
*             AZN_ENT_PDPOBJ_OUT_OF_MEMORY
*             AZN_ENT_PDPOBJ_INVALID_ARGUMENT
*
* These return codes have been replaced by the following major return codes
* defined in ogauthzn.h
*             AZN_S_SVC_ENT_INVALID_SVCINFO_HDL
*             AZN_S_SVC_ENT_INVALID_ARG_COUNT
*             AZN_S_SVC_ENT_INVALID_ARG_ARRAY
*             AZN_S_SVC_ENT_OUT_OF_MEMORY
*             AZN_S_SVC_ENT_INVALID_ARGUMENT
*
**************************************************************************/

#define AZN_ENT_PDPOBJ_INVALID_SVCINFO_HDL       (azn_ent_svc_err_prefix | 0x1)
#define AZN_ENT_PDPOBJ_INVALID_ARG_COUNT         (azn_ent_svc_err_prefix | 0x2)
#define AZN_ENT_PDPOBJ_INVALID_ARG_ARRAY         (azn_ent_svc_err_prefix | 0x3)
#define AZN_ENT_PDPOBJ_OUT_OF_MEMORY             (azn_ent_svc_err_prefix | 0x4)
#define AZN_ENT_PDPOBJ_INVALID_ARGUMENT          (azn_ent_svc_err_prefix | 0x5)

/**************************************************************************
*               Deprecated DCE EPAC Service minor return codes.
*
* The following minor return codes have been deprecated:
*             AZN_PAC_EPAC_INVALID_SVCINFO_HDL
*             AZN_PAC_EPAC_INVALID_ARG_COUNT
*             AZN_PAC_EPAC_INVALID_ARG_ARRAY
*             AZN_PAC_EPAC_OUT_OF_MEMORY
*             AZN_PAC_EPAC_INVALID_ARGUMENT
*
* These return codes have been replaced by the following major return codes
* defined in ogauthzn.h
*             AZN_S_SVC_PAC_INVALID_SVCINFO_HDL
*             AZN_S_SVC_PAC_INVALID_ARG_COUNT
*             AZN_S_SVC_PAC_INVALID_ARG_ARRAY
*             AZN_S_SVC_PAC_OUT_OF_MEMORY
*             AZN_S_SVC_PAC_INVALID_ARGUMENT
*
**************************************************************************/

/* DCE EPAC service minor status codes  */
#define AZN_PAC_EPAC_INVALID_SVCINFO_HDL         (azn_pac_svc_err_prefix | 0x1)
#define AZN_PAC_EPAC_INVALID_ARG_COUNT           (azn_pac_svc_err_prefix | 0x2)
#define AZN_PAC_EPAC_INVALID_ARG_ARRAY           (azn_pac_svc_err_prefix | 0x3)
#define AZN_PAC_EPAC_OUT_OF_MEMORY               (azn_pac_svc_err_prefix | 0x4)
#define AZN_PAC_EPAC_INVALID_ARGUMENT            (azn_pac_svc_err_prefix | 0x5)


/**************************************************************************
*                     Deprecated aznAPIs
*
* The azn_creds_compare() API has been deprecated.
* It is replaced by the azn_creds_equal() API.
*
* The azn_creds_set_attr_value_string() API has been deprecated.
* Use azn_creds_modify() instead.
*
* The azn_creds_get_attr_value_string() API has been deprecated.
* Use azn_creds_get_attrlist_for_subject() API and
* azn_attrlist_get_entry_string_value() on the returned attrlist instead.
*
**************************************************************************/

AZN_EXPORT
azn_boolean_t
AZN_CALLTYPE
azn_creds_compare(
    const azn_creds_h_t cred1,  /* in */
    const azn_creds_h_t cred2   /* in */
);

AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_creds_get_attr_value_string(
  const azn_creds_h_t      creds            /* in */,
  const unsigned int       subject_index    /* in */,
  const azn_string_t       attr_name        /* in */,
  azn_string_t       *attr_value      /* out */
);

AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_creds_set_attr_value_string(
  const azn_creds_h_t      creds            /* in */,
  const unsigned int       subject_index    /* in */,
  const azn_string_t       attr_name        /* in */,
  const azn_string_t       attr_value       /* in */
);

/**************************************************************************
*                Deprecated Utility Functions
*
* azn_util_server_authenticate() and azn_util_client_authenticate()
* DCE Authentication APIs have been deprecated.
* If called, they will return AZN_S_UNIMPLEMENTED_FUNCTION.
* Instead, applications are authenticated as part of azn_initialize().
*
**************************************************************************/
AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_util_server_authenticate(
    const azn_string_t principal_name,
    const azn_string_t keytab_path
);

AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_util_client_authenticate(
    const azn_string_t principal_name,
    const azn_string_t password
);


#ifdef __cplusplus
}
#endif

#endif  /* _azn_deprecated_h */
