/*
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2001,2002,2003.
 * US Government Users Restricted Rights - Use, duplicaion or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * All Rights Reserved.
 *
 * Copyright (c) 1998, 1999, 2000 Tivoli Systems. All Rights Reserved.
 *
 * File Name: ogauthzn.h
 */

#ifndef _OGAUTHZN_H
#define _OGAUTHZN_H

#include <stddef.h>


/*
 * Function linkage modifiers for Win32
 */
#if defined(WIN32) || defined(_WIN32)

#undef AZN_CALLTYPE
#define AZN_CALLTYPE       __cdecl

#undef AZN_EXPORT
#if defined(AZNDLL)
#define AZN_EXPORT         __declspec( dllexport )
#else
#define AZN_EXPORT         __declspec( dllimport )
#endif

#undef AZN_EXTSPEC
#define AZN_EXTSPEC        AZN_EXPORT

#undef AZN_EXTSPEC_CONST
#define AZN_EXTSPEC_CONST  AZN_EXPORT extern const

#else /* WIN32 */

#undef AZN_CALLTYPE
#define AZN_CALLTYPE

#undef AZN_EXPORT
#define AZN_EXPORT

#undef AZN_EXTSPEC
#define AZN_EXTSPEC extern

#undef AZN_EXTSPEC_CONST
#define AZN_EXTSPEC_CONST  extern const

#endif /* WIN32 */

/*
 * Defined Constants
 */

/* for azn_boolean_t */
#ifndef TRUE
#define TRUE    1
#endif
#ifndef FALSE
#define FALSE   0
#endif

/* Permission codes returned by azn_decision_access_allowed*() */
#define  AZN_C_PERMITTED               0
#define  AZN_C_NOT_PERMITTED           1

#define  AZN_C_EMPTY_BUFFER            (NULL)
#define  AZN_C_NO_BUFFER               (NULL)

#define  AZN_C_INITIATOR_INDEX         0
#define  AZN_C_AUDIT_ID                "AZN_AUDIT_ID"
#define  AZN_C_REQUEST_TIME            "AZN_REQUEST_TIME"
#define  AZN_C_AUTHN_QUALITY           "AZN_AUTHN_QUALITY"
#define  AZN_C_REQUESTER_LOC           "AZN_REQUESTER_LOC"
#define  AZN_C_REQUEST_ROUTE_QOP       "AZN_REQUEST_ROUTE_QOP"
#define  AZN_C_VERSION                 "AZN_VERSION"
#define  AZN_C_INVALID_HANDLE          ((long)(NULL))
#define  AZN_C_AUTHN_LEVEL             "AUTHENTICATION_LEVEL"

/*
 * Default authorization weightings for participating entities
 * in the authorization process. If there was no weighting
 * specified in the service definition then the default EAS
 * weighting is one more than the core authzn engine weighting.
 * In this scenario any ruling for/against the decision by the
 * EAS will take priority over decisions from the core engine.
 */
#define  AZN_C_AUTHZN_ENGINE_WEIGHTING (100)
#define  AZN_C_DEFAULT_EAS_WEIGHTING   (AZN_C_AUTHZN_ENGINE_WEIGHTING + 1)


/* API Major Status Codes */

#define AZN_S_ERRCODE_MIN                        0x0000
#define AZN_S_COMPLETE                           0x0000
#define AZN_S_FAILURE                            0x0001
#define AZN_S_AUTHORIZATION_FAILURE              0x0002
#define AZN_S_INVALID_CREDS_HDL                  0x0003
#define AZN_S_INVALID_NEW_CREDS_HDL              0x0004
#define AZN_S_INVALID_ENTITLEMENTS_SVC           0x0005
#define AZN_S_INVALID_COMB_CREDS_HDL             0x0006
#define AZN_S_INVALID_MECHANISM_INFO             0x0007
#define AZN_S_INVALID_MECHANISM                  0x0008
#define AZN_S_INVALID_STRING_VALUE               0x0009
#define AZN_S_UNKNOWN_LABEL                      0x000a
#define AZN_S_INVALID_ADDED_CREDS_HDL            0x000b
#define AZN_S_INVALID_PROTECTED_RESOURCE         0x000c
#define AZN_S_INVALID_OPERATION                  0x000d
#define AZN_S_INVALID_PAC                        0x000e
#define AZN_S_INVALID_PAC_SVC                    0x000f
#define AZN_S_INVALID_PERMINFO_REF               0x0010
#define AZN_S_INVALID_MOD_FUNCTION               0x0011
#define AZN_S_INVALID_SUBJECT_INDEX              0x0012
#define AZN_S_UNIMPLEMENTED_FUNCTION             0x0013
#define AZN_S_INVALID_ATTRLIST_HDL               0x0014
#define AZN_S_INVALID_ATTR_NAME                  0x0015
#define AZN_S_INVALID_BUFFER                     0x0016
#define AZN_S_INVALID_BUFFER_REF                 0x0017
#define AZN_S_INVALID_STRING_REF                 0x0018
#define AZN_S_ATTR_VALUE_NOT_STRING_TYPE         0x0019
#define AZN_S_ATTR_INVALID_INDEX                 0x001a
#define AZN_S_INVALID_INTEGER_REF                0x001b
#define AZN_S_INVALID_PERMISSION_REF             0x001c
#define AZN_S_INVALID_AUTHORITY                  0x001d
#define AZN_S_INVALID_APP_CONTEXT_HDL            0x001e
#define AZN_S_INVALID_ENTITLEMENTS_HDL           0x001f
#define AZN_S_INVALID_LABELING_SCHEME            0x0020
#define AZN_S_INVALID_INIT_DATA_HDL              0x0021
#define AZN_S_INVALID_INIT_INFO_HDL              0x0022
#define AZN_S_ATTR_VALUE_NOT_BUFFER_TYPE         0x0023
#define AZN_S_API_UNINITIALIZED                  0x0024
#define AZN_S_API_ALREADY_INITIALIZED            0x0025
#define AZN_S_SVC_DEFINITION_ERROR               0x0026
#define AZN_S_SVC_SERVICE_NOT_FOUND              0x0027
#define AZN_S_SVC_INIT_FAILED                    0x0028
#define AZN_S_SVC_SHUTDOWN_FAILED                0x0029
#define AZN_S_SVC_AUTHORIZATION_FAILED           0x002A
#define AZN_S_SVC_DLL_LOAD_FAILED                0x002B
#define AZN_S_SVC_INITIALIZE_NOT_FOUND           0x002C
#define AZN_S_SVC_SHUTDOWN_NOT_FOUND             0x002D
#define AZN_S_SVC_ENT_FUNC_NOT_FOUND             0x002E
#define AZN_S_SVC_PAC_FUNC_NOT_FOUND             0x002F
#define AZN_S_SVC_EAS_FUNC_NOT_FOUND             0x0030
#define AZN_S_SVC_CRED_MOD_FUNC_NOT_FOUND        0x0031
#define AZN_S_SVC_SERVICE_IS_REGISTERED          0x0032
#define AZN_S_SVC_DISPATCHER_FAILURE             0x0033
#define AZN_S_MINOR_CODE_MESSAGE_NOT_FOUND       0x0034
#define AZN_S_INVALID_EAS_ACL_TRIGGER            0x0035
#define AZN_S_INVALID_EAS_POP_TRIGGER            0x0036
#define AZN_S_INVALID_EAS_WEIGHTING              0x0037
#define AZN_S_UNKNOWN_EAS_SVC_PARAMETER          0x0038
#define AZN_S_SVC_ADMIN_POBJ_FUNC_NOT_FOUND      0x0039
#define AZN_S_INVALID_POBJ                       0x003A
#define AZN_S_INVALID_POBJ_REF                   0x003B
#define AZN_S_ATTR_VALUE_NOT_POBJ_TYPE           0x003C
#define AZN_S_SVC_ADMIN_UNKNOWN_PARAMETER        0x003D
#define AZN_S_SVC_ADMIN_POBJ_NOT_SPECIFIED       0x003E
#define AZN_S_SVC_ADMIN_TASK_FUNC_NOT_FOUND      0x003F
#define AZN_S_SVC_ADMIN_POBJ_ALREADY_REGISTERED  0x0040
#define AZN_S_INVALID_MESSAGE_ID_REF             0x0041
#define AZN_S_MAJOR_CODE_MESSAGE_NOT_FOUND       0x0042
#define AZN_S_ATTR_VALUE_NOT_ULONG_TYPE          0x0043
#define AZN_S_SVC_ADMIN_INVALID_SVCINFO_HDL      0x0044
#define AZN_S_SVC_ADMIN_INVALID_ARG_COUNT        0x0045
#define AZN_S_SVC_ADMIN_INVALID_ARG_ARRAY        0x0046
#define AZN_S_SVC_ADMIN_OUT_OF_MEMORY            0x0047
#define AZN_S_SVC_ADMIN_INVALID_ARGUMENT         0x0048
#define AZN_S_SVC_ADMIN_INVALID_TASK             0x0049
#define AZN_S_SVC_ENT_INVALID_SVCINFO_HDL        0x004A
#define AZN_S_SVC_ENT_INVALID_ARG_COUNT          0x004B
#define AZN_S_SVC_ENT_INVALID_ARG_ARRAY          0x004C
#define AZN_S_SVC_ENT_OUT_OF_MEMORY              0x004D
#define AZN_S_SVC_ENT_INVALID_ARGUMENT           0x004E
#define AZN_S_SVC_EAS_INVALID_SVCINFO_HDL        0x004F
#define AZN_S_SVC_EAS_INVALID_ARG_COUNT          0x0050
#define AZN_S_SVC_EAS_INVALID_ARG_ARRAY          0x0051
#define AZN_S_SVC_EAS_OUT_OF_MEMORY              0x0052
#define AZN_S_SVC_EAS_INVALID_ARGUMENT           0x0053
#define AZN_S_SVC_CRED_MOD_INVALID_SVCINFO_HDL   0x0054
#define AZN_S_SVC_CRED_MOD_INVALID_ARG_COUNT     0x0055
#define AZN_S_SVC_CRED_MOD_INVALID_ARG_ARRAY     0x0056
#define AZN_S_SVC_CRED_MOD_OUT_OF_MEMORY         0x0057
#define AZN_S_SVC_CRED_MOD_INVALID_ARGUMENT      0x0058
#define AZN_S_SVC_PAC_INVALID_SVCINFO_HDL        0x0059
#define AZN_S_SVC_PAC_INVALID_ARG_COUNT          0x005A
#define AZN_S_SVC_PAC_INVALID_ARG_ARRAY          0x005B
#define AZN_S_SVC_PAC_OUT_OF_MEMORY              0x005C
#define AZN_S_SVC_PAC_INVALID_ARGUMENT           0x005D
#define AZN_S_INVALID_LISTENING_PORT             0x005E
#define AZN_S_INVALID_MAJOR_CODE                 0x005F
#define AZN_S_ATTR_READONLY                      0x0060
#define AZN_S_DOMAIN_CONFLICT                    0x0061
#define AZN_S_ERRCODE_MAX                        0x0061

/* Utility Function Major Status Codes */
#define AZN_S_U_ERRCODE_MIN                0x0100
#define AZN_S_U_INTERNAL_ERROR             0x0100 /* internal err: see minor */
#define AZN_S_U_DCE_AUTHEN_FAILED          0x0101 /* DCE authenticate failed */
#define AZN_S_U_LDAP_AUTHEN_FAILED         0x0102 /* LDAP authenticate failed*/
#define AZN_S_U_ALREADY_AUTHENTICATED      0x0103 /* API caller is logged in */
#define AZN_S_U_PASSWORD_EXPIRED           0x0104 /* user's password expired */
#define AZN_S_U_INVALID_USER               0x0105 /* user info is invalid */
#define AZN_S_U_USER_REGISTRY_UNAVAIL      0x0106 /* user registry is offline */
#define AZN_S_U_INVALID_PARAMETER          0x0107 /* invalid calling params */
#define AZN_S_U_PTHREAD_ERROR              0x0108 /* error from pthread call */
#define AZN_S_U_INVALID_PRINC_NAME         0x0109 /* invalid principal name */
#define AZN_S_U_INVALID_PASSWORD           0x010a /* invalid password string */
#define AZN_S_U_INVALID_MECH_ID_REF        0x010b /* mech id ref is invalid */
#define AZN_S_U_INVALID_KEYTAB_PATH        0x010c /* invalid keytab string */
#define AZN_S_U_UNUSED_010D                0x010d /* unused */
#define AZN_S_U_ACCOUNT_DISABLED           0x010e /* account login disabled */
#define AZN_S_U_TOD_ACCESS_DENIED          0x010f /* Time Of Day access denied */
#define AZN_S_U_ACCOUNT_LOCKEDOUT          0x0110 /* Too many bad login attempts */
#define AZN_S_U_PASSWORD_TOO_SHORT         0x0111 /* new password to short */
#define AZN_S_U_PASSWORD_HAS_SPACES        0x0112 /* new password has illegal spaces */
#define AZN_S_U_PASSWORD_TOO_MANY_REPEATED 0x0113 /* new password has too many repeated characters */
#define AZN_S_U_PASSWORD_TOO_FEW_ALPHA     0x0114 /* new password has too few alphabetic characters  */
#define AZN_S_U_PASSWORD_TOO_FEW_NONALPHA  0x0115 /* new password has too few non-alphabetic characters */
#define AZN_S_U_INSUFFICIENT_ACCESS        0x0116 /* don't have the rights to do the operation */
#define AZN_S_U_URAF_AUTHEN_FAILED         0x0117 /* URAF authenticate failed */
#define AZN_S_U_PASSWORD_ACCT_DISABLED     0x0118 /* account disabled due to too many failed logins */
#define AZN_S_U_ACCOUNT_EXPIRED            0x0119 /* account expiration date has passed*/
#define AZN_S_U_ERRCODE_MAX                0x0119


/* Attribute list value types returned by azn_attrlist_get_entry_type() */
#define AZN_VALUE_TYPE_NONE                     0
#define AZN_VALUE_TYPE_STRING                   1
#define AZN_VALUE_TYPE_BUFFER                   2
#define AZN_VALUE_TYPE_POBJ                     3
/*      Reserved                                4                       */
#define AZN_VALUE_TYPE_ULONG                    5
#define AZN_VALUE_TYPE_MAX                      5


/* PD Protected Object entitlement service minor status codes  */
#define AZN_ENT_PDPOBJ_UNUSED_COMPATIBILITY_1    (azn_ent_svc_err_prefix | 0x1)
#define AZN_ENT_PDPOBJ_UNUSED_COMPATIBILITY_2    (azn_ent_svc_err_prefix | 0x2)
#define AZN_ENT_PDPOBJ_UNUSED_COMPATIBILITY_3    (azn_ent_svc_err_prefix | 0x3)
#define AZN_ENT_PDPOBJ_UNUSED_COMPATIBILITY_4    (azn_ent_svc_err_prefix | 0x4)
#define AZN_ENT_PDPOBJ_UNUSED_COMPATIBILITY_5    (azn_ent_svc_err_prefix | 0x5)
#define AZN_ENT_PDPOBJ_INVALID_OBJECT_PATH       (azn_ent_svc_err_prefix | 0x6)
#define AZN_ENT_PDPOBJ_INVALID_OPERATION_SET     (azn_ent_svc_err_prefix | 0x7)
#define AZN_ENT_PDPOBJ_INVALID_NUM_OPERATION_SET (azn_ent_svc_err_prefix | 0x8)
#define AZN_ENT_PDPOBJ_AUTHZN_ENGINE_NOT_FOUND   (azn_ent_svc_err_prefix | 0x9)
#define AZN_ENT_PDPOBJ_AZN_CLIENT_UNSUPPORTED    (azn_ent_svc_err_prefix | 0xa)


/* Type Definitions */

/* implementation specific status and handle definitions */

typedef unsigned long azn_status_t;
typedef long azn_creds_h_t;
typedef long azn_attrlist_h_t;
typedef unsigned long azn_boolean_t;
typedef unsigned long azn_ulong_t;

/* standard data types */

typedef struct azn_buffer_desc_struct
{
  size_t         length;
  unsigned char  *value;
} azn_buffer_desc, *azn_buffer_t;

typedef char *azn_string_t;

typedef struct azn_pobj_desc_struct
{
  azn_string_t    name; /* protected object path */
  unsigned int    type; /* protected object type; customer defined */
  azn_string_t    description; /* protected object description */
  azn_boolean_t   is_policy_attachable; /* indicates whether authorization
                                       policy can be attached to this object */
} azn_pobj_desc, *azn_pobj_t;


/*
 * Unauthenticated user mechanism information structure.
 *
 * This structure is used by the Auth API as an input of credential
 * and other information for use within the Policy Director secure
 * domain.
 *
 * Fields:
 *
 *      ipaddr          IP address of requesting user in network
 *                      byte order.
 *
 *      qop             Quality of protection required for requests
 *                      made by this user.
 *
 *      user_info       Additional user information that may be
 *                      required for auditing.
 *
 *      browser_info    The browser (if any) employed by the user.
 *
 */
typedef struct
{
    unsigned int        ipaddr;
    azn_string_t        qop;
    azn_string_t        user_info;
    azn_string_t        browser_info;
} azn_unauth_t;


/*
 * Default user registry mechanism information structure.
 *
 * This structure is used by the Auth API as an input of credential
 * and other information for use within the Policy Director secure
 * domain. The default user registry is used to build the client
 * credential. This structure can be used with a NULL mechanism ID
 * in a call to azn_id_get_creds() and the client credential will
 * be built from the default user registry with which PD was installed.
 * The client does not need to know what the user registry type is.
 *
 * Fields:
 *
 *      principal       The client principal's name in the registry.
 *
 *      ipaddr          IP address of requesting user in network
 *                      byte order.
 *
 *      qop             Quality of protection required for requests
 *                      made by this user.
 *
 *      user_info       Additional user information that may be
 *                      required for auditing.
 *
 *      browser_info    The browser (if any) employed by the user.
 *
 *      authnmech_info  Additional authentication mech info.
 *
 *  reserved    Fields reserved for future use.
 *
 */
typedef struct
{
    azn_string_t        principal;
    azn_string_t        auth_method;
    unsigned int        ipaddr;
    azn_string_t        qop;
    azn_string_t        user_info;
    azn_string_t        browser_info;
    azn_string_t        authnmech_info;
    void        *reserved[9];
} azn_authdefault_t;


/*
 * DCE specific authentication mechanism information structure.
 *
 * This structure is used by the Auth API as an input of credential
 * and other information for use within the Policy Director secure
 * domain.
 *
 * Fields:
 *
 *      context         A DCE login context pointer.
 *
 *      ipaddr          IP address of requesting user in network
 *                      byte order.
 *
 *      qop             Quality of protection required for requests
 *                      made by this user.
 *
 *      user_info       Additional user information that may be
 *                      required for auditing.
 *
 *      browser_info    The browser (if any) employed by the user.
 *
 *      authnmech_info  Additional authentication mech info.
 *
 */
typedef struct
{
    azn_string_t        principal;
    azn_string_t        auth_method;
    unsigned int        ipaddr;
    azn_string_t        qop;
    azn_string_t        user_info;
    azn_string_t        browser_info;
    azn_string_t        authnmech_info;
} azn_authdce_t;


/*
 * LDAP specific authentication mechanism information structure.
 *
 * This structure is used by the Auth API as an input of credential
 * and other information for use within the Policy Director secure
 * domain.
 *
 * Fields:
 *
 *      ldap_dn         An LDAP distinguished name.
 *
 *      ipaddr          IP address of requesting user in network
 *                      byte order.
 *
 *      qop             Quality of protection required for requests
 *                      made by this user.
 *
 *      user_info       Additional user information that may be
 *                      required for auditing.
 *
 *      browser_info    The browser (if any) employed by the user.
 *
 *      authnmech_info  Additional authentication mech info.
 *
 *
 */
typedef struct
{
    azn_string_t        ldap_dn;
    azn_string_t        auth_method;
    unsigned int        ipaddr;
    azn_string_t        qop;
    azn_string_t        user_info;
    azn_string_t        browser_info;
    azn_string_t        authnmech_info;
} azn_authldap_t;


/*
 * URAF specific authentication mechanism information structure.
 *
 * This structure is used by the Auth API as an input of credential
 * and other information for use within the Policy Director secure
 * domain.
 *
 * Fields:
 *
 *      uraf_user       An URAF user name.
 *
 *      ipaddr          IP address of requesting user in network
 *                      byte order.
 *
 *      qop             Quality of protection required for requests
 *                      made by this user.
 *
 *      user_info       Additional user information that may be
 *                      required for auditing.
 *
 *      browser_info    The browser (if any) employed by the user.
 *
 *      authnmech_info  Additional authentication mech info.
 *
 *  uraf_handle Pointer to a URAF registry handle context.
 *
 *
 */
typedef struct
{
    azn_string_t        uraf_user;
    azn_string_t        auth_method;
    unsigned int        ipaddr;
    azn_string_t        qop;
    azn_string_t        user_info;
    azn_string_t        browser_info;
    azn_string_t        authnmech_info;
    void                *uraf_handle;
} azn_authuraf_t;



#ifdef __cplusplus
extern "C" {
#endif

/**********************************************************************
 * Constant variable declarations
 **********************************************************************/
/*
 * Mechanism IDs for azn_id_get_cred(). Each of these match
 * one of the azn_auth*_t type definitions above.
 */
#if !defined(AZNDLL)
AZN_EXTSPEC azn_string_t IV_UNAUTH; /* unauthenticated user */
AZN_EXTSPEC azn_string_t IV_DCE;    /* DCE authenticated user */
AZN_EXTSPEC azn_string_t IV_LDAP;   /* LDAP authenticated user */
AZN_EXTSPEC azn_string_t IV_URAF;   /* URAF authenticated user */
#endif



/**************************************************************************
 * Operation string declarations. These values are passed into the
 * azn_decision_access_allowed* and azn_entitlements_get_operations*
 * calls respectively. The actual string values of these operations
 * are an internal implementation detail and should not be relied
 * upon. The operations can be concatenated together to form complex
 * operation strings e.g., to request a read/modify operation.
 */
AZN_EXTSPEC_CONST azn_string_t azn_operation_attach;
AZN_EXTSPEC_CONST azn_string_t azn_operation_bypasspop;
AZN_EXTSPEC_CONST azn_string_t azn_operation_bypassrule;
AZN_EXTSPEC_CONST azn_string_t azn_operation_browse;
AZN_EXTSPEC_CONST azn_string_t azn_operation_control;
AZN_EXTSPEC_CONST azn_string_t azn_operation_traverse;
AZN_EXTSPEC_CONST azn_string_t azn_operation_delegation;
AZN_EXTSPEC_CONST azn_string_t azn_operation_view;
AZN_EXTSPEC_CONST azn_string_t azn_operation_modify;
AZN_EXTSPEC_CONST azn_string_t azn_operation_delete;
AZN_EXTSPEC_CONST azn_string_t azn_operation_server_admin;
AZN_EXTSPEC_CONST azn_string_t azn_operation_read;
AZN_EXTSPEC_CONST azn_string_t azn_operation_execute;
AZN_EXTSPEC_CONST azn_string_t azn_operation_list_directory;
AZN_EXTSPEC_CONST azn_string_t azn_operation_connect;
AZN_EXTSPEC_CONST azn_string_t azn_operation_forward;
AZN_EXTSPEC_CONST azn_string_t azn_operation_password;
AZN_EXTSPEC_CONST azn_string_t azn_operation_create;
AZN_EXTSPEC_CONST azn_string_t azn_operation_add;
AZN_EXTSPEC_CONST azn_string_t azn_operation_trace;



/**************************************************************************
 * Initialization data attribute names.
 *
 * Passed to azn_initialize. Each attribute name requires one or
 * more const azn_string_t values as outlined below.
 */

/*
 * AZN API authzn engine mode: "local" (using a policy cache replica)
 * or "remote" (making RPC requests to ivacld).
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_mode;

/*
 * Valid azn_string_t constants for use with th azn_init_set_code_set
 * function and with the azn_svc_init_code_set service init attribute.
 */
    AZN_EXTSPEC_CONST azn_string_t azn_code_set_utf8;
    AZN_EXTSPEC_CONST azn_string_t azn_code_set_local;

/*
 * AZN API authzn server name
 * First appears in PD3.8.
 */
AZN_EXTSPEC_CONST azn_string_t azn_server_name;

/*
 * AZN API application host
 */
AZN_EXTSPEC_CONST azn_string_t azn_app_host;

/*
 * Path and filename to be used to contain remaining initialization
 * parameters.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_cfg_file;


/*
 * SSL attributes that control the communications between azn calls and
 * servers.  The units for azn_init_ssl_timeout are seconds.  The
 * units for azn_init_pwd_life are days.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_keyfile;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_stashfile;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_timeout;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_pwd_life;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_mgr_config;

/* First appearance in PD3.8. */
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_keyfile_pwd;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_keyfile_label;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_listening_port;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_cert_life;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_max_worker_threads;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_auto_refresh;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_io_inactivity_timeout;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_authn_type;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_authn_user;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_authn_pwd;

/* First appearance in AM 4.2 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_local_domain;


/*
 * Communications attributes, the azn_init_master_xxx attributes refer
 * to the location of ivmgrd, the replica attribute is the location of
 * an ivacld server.  There can be more than one azn_init_replica.
 *
 * The ivacld server replica's are of the format:
 *     <replica hostname>:<port>:<preference>:<replica cert dn>
 * where:
 *   <replica hostname> is the network name of the server.
 *   <port>             is the port on the server.
 *   <preference>       is a ranking for attempting contact from 1 to
 *                      10.  10 is the highest preference.
 *   <DN>               DEPRECATED
 * For example,
 *  "rweber.bball.com:7137:5:"
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_master_host;
AZN_EXTSPEC_CONST azn_string_t azn_init_master_port;
AZN_EXTSPEC_CONST azn_string_t azn_init_replica;


/*
 * The set of attributes that the caller is interested in receiving
 * from the azn_decision_access_allowed_ext() call in the permission
 * info attribute list. By default there is no information returned
 * by this call. Adding a value to this attribute at init time enables
 * the attribute to be returned as permission info if it is applicable
 * to the current decision call. This list may also include attributes
 * that are user defined. e.g., by setting an attribute on an ACL or POP
 * using "modify set attribute" command of "pdadmin".
 *
 * The caller can set values for this attribute in the init_data parameter 
 * to the azn_initialize() call, OR set them in the 'permission-info-returned' 
 * configuration file entry.
 *
 * See the section "Permission info attributes" below for the list of
 * string constant attribute names the caller can set for this init
 * attribute or config entry. The name of the attribute in the returned 
 * 'perminfo' list parameter of the azn_decision_access_allowed_ext() API 
 * will also match the permission info attribute name requested.
 *
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_set_perminfo_attrs;


/* Path and filename to be used to contain cached authzn policy. */
AZN_EXTSPEC_CONST azn_string_t azn_init_db_file;

/* String ID or tag to identify the client aznAPI client. Default: aznAPI. */
AZN_EXTSPEC_CONST azn_string_t azn_init_logging_client_id;

/* Audit log path and filename to collect PD audit events. */
AZN_EXTSPEC_CONST azn_string_t azn_init_audit_file;

/* Audit additional cred or application specific attribute */
AZN_EXTSPEC_CONST azn_string_t azn_init_audit_attribute;

/* Debug log path and filename to collect PD debug events. */
AZN_EXTSPEC_CONST azn_string_t azn_init_debug_file;

/*
 * Audit logging config.
 *
 * Multi-valued attribute containing entries of the format:
 *
 *  module-id[:trace-level]
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_auditcfg;

/* Debug logging config.
 *
 * Multi-valued attribute containing entries of the format:
 *
 *  module-id[:trace-level]
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_debugcfg;

/*
 * The maximum size (in bytes) for the audit and debug files. When
 * the log size reaches this value the log is backed up and a new
 * log file is started. A value of 0 will disable log rollover.
 * A negative value will perform rollover daily regardless of size.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_log_size;

/*
 * The audit/debug logging service flush interval in seconds.
 * 0 value sets default of 20 seconds. Maximum: 3600 (6 hours).
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_log_flush_interval;

/*
 * Interval in seconds for local policy cache polled updates. Values
 * can be "disable", "default" or a string time in seconds.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_cache_refresh_interval;

/*
 * Flags to enable the reception of policy cache update notifications.
 * Values can be a combination of: "disable", "enable", "use_tcp_port",
 * "use_udp_port" and "dynamic_port_selection". Multiple values are
 * accepted for this attribute name and are logically OR'd together.
 * A "disable" value overrides all others and disables the notification
 * listener.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_listen_flags;

/*
 * LDAP user registry configurations can specify the following
 * attributes to enable LDAP registry use in the AZN API.
 */

/* The LDAP server host name.  */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_host;

/*
 * The LDAP server host port (a numerical string). This will  be
 * the SSL port if SSL comms with the LDAP host is to be enabled.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_port;

/* The distinguished name to bind to the LDAP server with. */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_bind_dn;

/* The password for the bind DN. */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_bind_pwd;

/*
 * The LDAP server's SSL keyfile. (This must be non-NULL to use
 * SSL comms with the LDAP host)
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_ssl_keyfile;

/* The LDAP server's SSL keyfile distinguished name.  */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_ssl_keyfile_dn;

/* The LDAP server's SSL keyfile password.  */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_ssl_keyfile_pwd;

/*
 * The LDAP server replica's.
 *
 * Each string value is of the format <ldap-server>,<port>,<type>,<pref>
 * where:
 *   <ldap-server> is the network name of the server.
 *   <port> is the port on the ldap server.
 *   <type> is one of "readonly" or "readwrite"
 *   <pref> is a level from 1 to 10.  10 is the highest preference.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_replica;


/*
 * This enables caching of the user, group, and LDAP policy data in
 * the client to reduce the turnaround of similar LDAP transactions.
 * The string value can be "true" or "false". Defaults to "false".
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_cache;

/*
 * This makes the writable LDAP server the preferred one for
 * the client. Calls will still failover to a read-only server
 * if the write server is offline. The string value can be
 * "true" or "false". Defaults to "false".
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_prefer_rw_svr;

/*
 * Set the maximum number of entries returned from an LDAP search.
 * This is further constrained by the limits set by the LDAP server.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_max_search_size;


/*
 * Chooses whether the client will use the ldap_compare or the
 * ldap_bind call to authenticate a the users password. This
 * applies to calls to azn_util_client_authenticate() and
 * azn_util_password_authenticate() with an LDAP registry. The
 * string value can be "true" or "false". Default is "false"
 * which is to use the ldap_bind call.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_use_compare;

/*
 * Client side timeout settings for communications to
 * LDAP servers.
 *
 * The azn_init_ldap_timeout parameter is an overall
 * setting. This may be overridden for authentication
 * (bind and compare) and search by specifying the
 * the azn_init_ldap_authn_timeout and/or the
 * azn_init_ldap_search_timeout parameters respectively.
 *
 * Each of these timeouts is specified in seconds as
 * a string value.
 *
 * By default, or with a value of "0", LDAP requests
 * will not timeout.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_timeout;
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_authn_timeout;
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_search_timeout;

/*
 * URAF user registry configurations can specify the following
 * attributes to enable URAF registry use in the AZN API.
 * Started in AM 4.2 CMVC 28136
 */

/* uraf registry config file name */
AZN_EXTSPEC_CONST azn_string_t azn_init_uraf_cfg_file;
/* uraf registry bind ID */
AZN_EXTSPEC_CONST azn_string_t azn_init_uraf_bind_id;
/* uraf registry bind password */
AZN_EXTSPEC_CONST azn_string_t azn_init_uraf_bind_pwd;
/* enable or disable uraf cache, default value is disabled */
AZN_EXTSPEC_CONST azn_string_t azn_init_uraf_cache_mode;
/* uraf cache size, default value is 251 */
AZN_EXTSPEC_CONST azn_string_t azn_init_uraf_cache_size;
/* uraf cache lifetime, default value is 86400 sec. (one day) */
AZN_EXTSPEC_CONST azn_string_t azn_init_uraf_cache_lifetime;

/*
 * aznAPI service initialization attributes.
 *
 * Multi-valued string attributes for use with azn_initialize().
 * Each string value is of the format:
 *
 *  "<service-id> = <path-to-dll> [ & params ... ]"
 *
 * The service-id is the string by which the service can be
 * identified by the aznAPI client. The path-to-dll is the
 * fully qualified path to the DLL which contains the service
 * executable code. Optionally you can specify parameters to
 * pass to the service when it is initialised by the aznAPI.
 * The parameters are considered to be all data following the
 * '&' symbol in the string.
 *
 * The external authorization service can take an additional
 * optional "-weight" parameter as follows:
 *
 *      <service-id> = <path-to-dll> [ -weight N ] [ & params ... ]
 *
 * The value N can be any unsigned long value whose absolute
 * value is used to appropriately weight the decision of an
 * EAS for or against access to the object. The default value
 * for an EAS if this is not specified is set to the value
 * AZN_C_DEFAULT_EAS_WEIGHTING. The assigned weighting of the
 * core authorization engine is AZN_C_AUTHZN_ENGINE_WEIGHTING.
 */

/* Entitlements service definition attribute */
AZN_EXTSPEC_CONST azn_string_t azn_init_ent_svc;

/* PAC service definition attribute */
AZN_EXTSPEC_CONST azn_string_t azn_init_pac_svc;

/* Credential modification service definition attribute */
AZN_EXTSPEC_CONST azn_string_t azn_init_cred_mod_svc;

/* Administration service definition attribute */
AZN_EXTSPEC_CONST azn_string_t azn_init_admin_svc;

/* External Authorization Service definition attribute */
AZN_EXTSPEC_CONST azn_string_t azn_init_extern_authzn_svc;


/*
 *************************************************************
 *      aznAPI boolean rules evaluator init attributes
 *************************************************************
 */

/*
 * XML prolog text for XMLADI input document.
 *
 * This text is appended to the XML input document that is
 * created from the rule ADI passed in the application
 * context and credential attributes. The default text is:
 *
 *      <?xml version="1.0" encoding="UTF-8"?>
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_input_adi_xml_prolog;


/*
 * XMLADI attribute definitions.
 *
 * These attribute definitions are inserted into the XMLADI element
 * start tag to enable attributes to be defined for the entire XMLADI
 * docuement and all ADI defined therein. The programmatic initialization
 * attribute can contain multiple attribute values, one for each
 * attribute definition to be added to the XMLADI element start tag.
 * The attribute definitions are in the format:
 *
 *  AttributeName = "AttributeValue"
 *
 * AttributeValue must be enclosed in either double or single quotes.
 *
 * eg:
 *  xmlns:myNS = "http://myURI.mycompany.com"
 *  appID = '"Jupiter" - Account Management Web Portal Server #1.'
 *
 * The XMLADI element start tag built from these definitions is:
 *
 *  <XMLADI xmlns:myNS="http://myURI.mycompany.com" \
 *      appID='"Jupiter" - Account Management Web Portal Server #1.'>
 *
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_xmladi_attribute_definitions;


/*
 * XSL prolog text for the XSL rule document.
 *
 * This text is appended to the XSL rule document that is
 * created from the rule text in the rule policy object.
 * This allows you to control aspects of the XSL evaluation
 * process. Modifying this value is not recommended as it
 * can affect the results returned by the processor and
 * make them unrecognizable by the rules policy evaluator.
 * The default and recommended setting for this prolog is:
 *
    <!-- Required for XSLT language -->
    <?xml version="1.0" encoding='UTF-8'?>
    <xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                    version="1.0">

    <!-- Required to constrain output of rule evaluation -->
    <xsl:output method="text" omit-xml-declaration="yes"
                encoding='UTF-8' indent="no" />

    <!-- Need this to ensure default text node printing is off -->
    <xsl:template match="text()"></xsl:template>
 *
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_xsl_stylesheet_prolog;


/*
 * Resource manager ADI prefix list.
 *
 * A list of string prefixes that identify Access Decision Information (ADI)
 * to be supplied by the resource manager (ie. the AM "blade" or aznAPI
 * client). This list tells the authorization engine that whenever it needs
 * ADI that has one of the prefixes listed, and the ADI is not available in
 * the credential or application context passed in with the access decision
 * call, that the engine should fail the access decision and request that
 * the resource manager retry the request and provide the required data in
 * the application context of the next request. aznAPI clients (blades)
 * such as "Access Manager for E-Business" use this configuration entry to
 * notify the authorization engine that it is a source for HTTP transaction
 * data needed for rule evaluations.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_resource_mgr_provided_adi;


/*
 * Dynamic ADI Retrieval Entitlements Service List.
 *
 * A list of configured azn entitlements service IDs that will be queried
 * by the rules engine if missing ADI is detected during an authorization
 * rule evaluation. Entitlements services listed here must adhere to the
 * input and output specifications for a dynamic ADI retrieval service which
 * are outlined in the Authorization Programmer's.Reference. When ADI is
 * found to be missing during a rule evaluation each service in this list
 * is queried in the order defined in this entry. These entries must refer
 * to existing entitlements services that were loaded via service entries
 * in the entitlement service configuration stanza or initialization
 * attribute.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_dynamic_adi_entitlement_services;


/*
 * Credential Extended Attribute Entitlements Service List.
 *
 * A list of configured azn entitlements service IDs that will be queried
 * by azn_id_get_creds after the credential is built in order to gather
 * attributes to be placed in the credential for attribute-based authorization.
 * Each service ID is called according to the order of the values in the
 * azn_init_cred_attribute_entitlement_services initialization attribute, or
 * the order of values as defined in the cred-attribute-entitlement-services 
 * entry in the configuration file. 
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_cred_attribute_entitlement_services;



/**************************************************************************
 * Authzn credential attribute names.
 *
 * These attributes are returned by the azn_creds_get_attrlist_for_subject
 * call. Some of these may not be present according to the type of the
 * authenticated user and the information that was originally passed to
 * the call used to build a given cred.
 */

/* the credential version */
AZN_EXTSPEC_CONST azn_string_t azn_cred_version;

/* the mechanism ID for this cred */
AZN_EXTSPEC_CONST azn_string_t azn_cred_mech_id;

/* the UUID of the entity */
AZN_EXTSPEC_CONST azn_string_t azn_cred_principal_uuid;

/* the string name of the entity */
AZN_EXTSPEC_CONST azn_string_t azn_cred_principal_name;

/* The local domain that the user is authenticated in. */
AZN_EXTSPEC_CONST azn_string_t azn_cred_principal_domain;

/* the string group UUID memberships of this entity */
AZN_EXTSPEC_CONST azn_string_t azn_cred_group_uuids;

/* the string group name memberships of this entity */
AZN_EXTSPEC_CONST azn_string_t azn_cred_group_registry_ids;
AZN_EXTSPEC_CONST azn_string_t azn_cred_groups;


/* the authentication ID used to build these authzn credentials */
AZN_EXTSPEC_CONST azn_string_t azn_cred_authzn_id;

/* any user info passed in the mech info struct */
AZN_EXTSPEC_CONST azn_string_t azn_cred_user_info;

/* any authn method info passed in the mech info struct */
AZN_EXTSPEC_CONST azn_string_t azn_cred_auth_method;

/* any authn mechanism info passed in the mech info struct */
AZN_EXTSPEC_CONST azn_string_t azn_cred_authnmech_info;

/* the IP address information passed in the mech info struct */
AZN_EXTSPEC_CONST azn_string_t azn_cred_ip_address;

/*
 * The Quality of Protection information passed in the mech info struct
 * One of "none", "integrity" or "privacy".
 */
AZN_EXTSPEC_CONST azn_string_t azn_cred_qop_info;

/* the browser information passed in the mech info struct */
AZN_EXTSPEC_CONST azn_string_t azn_cred_browser_info;

/* the registry ID used to build these authzn credentials */
AZN_EXTSPEC_CONST azn_string_t azn_cred_registry_id;



/**************************************************************************
 * Permission info attributes.
 *
 * These attributes are returned by the azn_decision_access_allowed_ext
 * call. Some of these may not be present in all permission_info attribute
 * lists returned according to the type of the access decision and the
 * information that was originally passed in with the call.
 *
 * These attribute names may also be passed in to the azn_initialize()
 * call as values for the azn_init_set_perminfo_attrs attribute to control which
 * of them are returned as permission info for any decision call. The
 * default is that no permission information will be returned.
 */

/*
 * This value is passed in the azn_init_set_perminfo_attrs attribute
 * to azn_initialize() to specify that the caller wished to receive
 * all attributes that can potentially be returned by the authzn
 * engine on an extended authzn decision call. This will override
 * any other attributes passed in the azn_init_set_perminfo_attrs
 * attribute.
 */
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_all_attrs;

/*
 * The quality of protection required between caller and the domain
 * before access to the protected object can be granted. The QOP value
 * will be one of "none", "integrity" or "privacy".
 *
 *  The caller needs to use azn_attrlist_add_entry  to set this value in
 *  the azn_init_set_perminfo_attrs attribute in the init_data parameter to
 *  the azn_initialize() call.
 *
 *  The caller needs to use the azn_attrlist_get_entry() API to
 *  retrieve this attribute from the perm_info output parameter of the
 *  azn_decision_access_allowed_ext() API.
 *
 * type: const azn_string_t
 */
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_qop;

/*
 * This is the same as the azn_perminfo_qop attribute except it is returned as
 * an unsigned long.
 *
 *  The caller needs to use azn_attrlist_add_entry  to set this value in
 *  the azn_init_set_perminfo_attrs attribute in the init_data parameter to
 *  the azn_initialize() call.
 *
 *  The caller needs to use the azn_attrlist_get_entry_ulong() API to
 *  retrieve this attribute from the perm_info output parameter of the
 *  azn_decision_access_allowed_ext() API.
 *
 * type:  unsigned long
 */
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_qop_ulong;

/* azn_perminfo_qop_ulong values */
#if !defined(AZNDLL)
AZN_EXTSPEC_CONST unsigned long azn_perminfo_qop_value_none;
AZN_EXTSPEC_CONST unsigned long azn_perminfo_qop_value_integrity;
AZN_EXTSPEC_CONST unsigned long azn_perminfo_qop_value_privacy;
#endif

/*
 * The authentication level required to access the object, returned as an
 * unsigned long.
 *
 *  The caller needs to use azn_attrlist_add_entry  to set this value in
 *  the azn_init_set_perminfo_attrs attribute in the init_data parameter to
 *  the azn_initialize() call.
 *
 *  The caller needs to use the azn_attrlist_get_entry_ulong() API to
 *  retrieve this attribute from the perm_info output parameter of the
 *  azn_decision_access_allowed_ext() API.
 *
 * type:  unsigned long
 */
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_stepup_level;

/*
 * Warning mode.  When warning mode is enabled access is always granted.
 * If access should not have been granted then the access is logged.
 *
 *  The caller needs to use azn_attrlist_add_entry  to set this value in
 *  the azn_init_set_perminfo_attrs attribute in the init_data parameter to
 *  the azn_initialize() call.
 *
 *  The caller needs to use the azn_attrlist_get_entry_ulong() API to
 *  retrieve this attribute from the perm_info output parameter of the
 *  azn_decision_access_allowed_ext() API.
 *
 * type:  boolean values returned in unsigned long
 */

AZN_EXTSPEC_CONST azn_string_t azn_perminfo_warningmode_ulong;

/*
 * Access permitted because of warning mode.  The boolean indicator is used
 * to tell the caller that access was granted because warning mode is enabled.
 *
 *  The caller needs to use azn_attrlist_add_entry  to set this value in
 *  the azn_init_set_perminfo_attrs attribute in the init_data parameter to
 *  the azn_initialize() call.
 *
 *  The caller needs to use the azn_attrlist_get_entry_ulong() API to
 *  retrieve this attribute from the perm_info output parameter of the
 *  azn_decision_access_allowed_ext() API.
 *
 * type:  boolean values returned in unsigned long
 */
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_warningmodepermitted_ulong;

/*
 * Audit level.  Audit events that are performed for this authorization check.
 *
 *  The caller needs to use azn_attrlist_add_entry  to set this value in
 *  the azn_init_set_perminfo_attrs attribute in the init_data parameter to
 *  the azn_initialize() call.
 *
 *  The caller needs to use the azn_attrlist_get_entry_ulong() API to
 *  retrieve this attribute from the perm_info output parameter of the
 *  azn_decision_access_allowed_ext() API.
 *
 * type: unsigned long bitmask
 */

AZN_EXTSPEC_CONST azn_string_t azn_perminfo_auditlevel_ulong;

/* azn_perminfo_auditlevel_ulong values */

#if !defined(AZNDLL)
AZN_EXTSPEC_CONST unsigned long azn_perminfo_auditlevel_all;
AZN_EXTSPEC_CONST unsigned long azn_perminfo_auditlevel_none;
AZN_EXTSPEC_CONST unsigned long azn_perminfo_auditlevel_permit;
AZN_EXTSPEC_CONST unsigned long azn_perminfo_auditlevel_deny;
AZN_EXTSPEC_CONST unsigned long azn_perminfo_auditlevel_error;
AZN_EXTSPEC_CONST unsigned long azn_perminfo_auditlevel_admin;
#endif

/*
 * POP/ACL location perminfo
 *
 *  The caller needs to use azn_attrlist_add_entry  to set this value in
 *  the azn_init_set_perminfo_attrs attribute in the init_data parameter to
 *  the azn_initialize() call.
 *
 *  The caller needs to use the azn_attrlist_get_entry() API to
 *  retrieve this attribute from the perm_info output parameter of the
 *  azn_decision_access_allowed_ext() API.
 *
 */
AZN_EXTSPEC_CONST azn_string_t azn_pop_ext_attr_loc;
AZN_EXTSPEC_CONST azn_string_t azn_acl_ext_attr_loc;
AZN_EXTSPEC_CONST azn_string_t azn_rule_ext_attr_loc;


/*
 *************************************************************
 *      aznAPI boolean rules evaluator perminfo attributes
 *************************************************************
 */

/*
 * A list of string ADI container names that the rules evaluator
 * needs in order to evaluate the current rule. Upon receiving
 * this attribute in the permission info from an access decision
 * call the resource manager should collate the request containers
 * of data, add them to the app_context parameter and call the
 * access decision interface again.
 */
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_rules_adi_request;

/*
 * A string indicating the reason that a rule failed. This is
 * returned in the permission info from an access decision call
 * when the rule failed and a reason string was set on the rule
 * by the policy administrator. The string is user-specific and
 * not mandatory.
 */
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_reason_rule_failed;

/*
 * WebSEAL/AMWebARS specific. The URL that WebSEAL should redirect
 * the user brower to in the event that the dynamic ADI retrieval
 * module needs to interact with the user before ADI can be
 * retrieved. Refer to the WebSEAL documentation for more info.
 */
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_dynadi_redirect_url;

/*
 * WebSEAL/AMWebARS specific. The full target URL that the caller
 * is requesting. This is required by AMWebARS in order to enable
 * it to return the browser to the original requested URL once
 * its interaction with the user is complete. This attribute
 * is passed into the access decision by WebSEAL. Refer to the
 * WebSEAL documentation for more info.
 */
AZN_EXTSPEC_CONST azn_string_t azn_dynadi_target_url;


/* These attributes may returned by the azn_decision_access_allowed_ext
 * call.  The azn_perminfo_fail_reason may contain one of succeeding
 * reason codes.
 */
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_fail_reason;

AZN_EXTSPEC_CONST unsigned long azn_perminfo_fail_reason_acl;
AZN_EXTSPEC_CONST unsigned long azn_perminfo_fail_reason_traverse;
AZN_EXTSPEC_CONST unsigned long azn_perminfo_fail_reason_tod;
AZN_EXTSPEC_CONST unsigned long azn_perminfo_fail_reason_eas;
AZN_EXTSPEC_CONST unsigned long azn_perminfo_fail_reason_eas_override;
AZN_EXTSPEC_CONST unsigned long azn_perminfo_fail_reason_stepup_forbidden;
AZN_EXTSPEC_CONST unsigned long azn_perminfo_fail_reason_delegate;


/**********************************************************************
 * aznAPI service module data and attribute names.
 **********************************************************************/

/* The prefixes used by the services to define minor error codes. */
#if !defined(AZNDLL)
AZN_EXTSPEC unsigned int azn_ent_svc_err_prefix;
AZN_EXTSPEC unsigned int azn_mod_svc_err_prefix;
AZN_EXTSPEC unsigned int azn_pac_svc_err_prefix;
AZN_EXTSPEC unsigned int azn_eas_svc_err_prefix;
AZN_EXTSPEC unsigned int azn_admin_svc_err_prefix;
#endif

/*
 * Used to return the version string of a service module to
 * the aznAPI runtime.
 */
AZN_EXTSPEC_CONST azn_string_t azn_svc_version;

/*
 * Returns any plug-in load errors returned by the system in
 * the format:
 *
 *  "dll-path: error-string"
 *
 * Where the dll-path is the filename of the DLL that was passed
 * to the load routine and error-string is the result returned.
 */
AZN_EXTSPEC_CONST azn_string_t azn_svc_load_error;

/*
 * Used by the service dispatcher to pass the service ID for a
 * service module to the modules initialization routine.
 */
AZN_EXTSPEC_CONST azn_string_t azn_svc_init_my_service_id;

/*
 * Code set initalization attribute for the aznAPI services. 
 * If the attribute is not passed in the svcInit init attribute
 * list to azn_svc_initialize then the service should assume
 * that the aznAPI application is using the local codeset.
 * Otherwise the value of this attribute will equal one of the 
 * azn_string_t code set name constants (eg 'azn_code_set_utf8')
 * defined above.
 */
    AZN_EXTSPEC_CONST azn_string_t azn_svc_init_code_set;


/**********************************************************************
 * Credential Modification Service Attributes.
 **********************************************************************/

/*
 * AM Credential Groups Modification Service.
 *
 * Effectively adds the credential to the list of groups
 * specified in the input attributes list. Note that the
 * service returns a new credential with this capability
 * and the original cred (and the registry) are unchanged.
 */
/* Mod svc attribute identifying the list of groups to add. */
AZN_EXTSPEC_CONST azn_string_t azn_mod_rad_group_names;

/* cred group modification service error codes */
#define AZN_MOD_GROUP_ADD_INVALID_SVCINFO_HDL  (azn_mod_svc_err_prefix | 0x1)
#define AZN_MOD_GROUP_ADD_INVALID_ARG_COUNT    (azn_mod_svc_err_prefix | 0x2)
#define AZN_MOD_GROUP_ADD_INVALID_ARG_ARRAY    (azn_mod_svc_err_prefix | 0x3)
#define AZN_MOD_GROUP_ADD_OUT_OF_MEMORY        (azn_mod_svc_err_prefix | 0x4)


/**********************************************************************
 * Credential Attributes Entitlements Service Attributes.
 **********************************************************************/

/*
 * A list of attributes requested by the application through app_context
 * of the Credential Attributes Entitlements Service.  The format of this   
 * string value is "registry_identifier:registry_attribute_name"
 * where the registry_identifier is the value used to do the registry
 * search.  This can also be a credential attribute name as long as the
 * the value resolves to a registry identifier.  The regsitry_attribute_name
 * is the attribute to be retrieved from the regsitry. Examples are:
 * "cn=myname,o=myorg:address"
 * where "cn=myname,o=myorg" is the registry identifier
 * and "address" is the attribute wanted from the registry
 * "azn_cred_authzn_id:EmpolyeeType"
 * where azn_cred_authzn_id is an attribute in the cred that is a DN
 * and EmployeeType is the attribute in the registry
 */
AZN_EXTSPEC_CONST azn_string_t azn_ent_cred_attrs_attribute;


/**********************************************************************
 * Entitlements service IDs.
 **********************************************************************/

/*
 * Entitlements Service ID: Policy Director Protected Object Entitlements.
 *
 * This service takes a credential, a directory within the protected
 * object tree and a set of operations and returns the list of protected
 * objects in the given directory and its subdirectories for which the
 * given aznAPI credential has permission to perform the requested set
 * of operations. The output is returned as a multi-string valued
 * attribute.
 *
 * NOTE: this is a default service. ie. when a NULL entitlements
 * service ID is specified to azn_entitlements_get_entitlements()
 * then this is the service that is called.
 */
AZN_EXTSPEC_CONST azn_string_t azn_ent_svc_pd_pobj;  /* service ID */

/* Input attribute: the protected object directory path to be searched */
AZN_EXTSPEC_CONST azn_string_t azn_ent_svc_pd_pobj_path;

/* Input attribute: the requested set of operations for the credential */
AZN_EXTSPEC_CONST azn_string_t azn_ent_svc_pd_pobj_reqd_ops;

/* Output attribute: the list of objects which match the input criteria */
AZN_EXTSPEC_CONST azn_string_t azn_ent_svc_pd_pobj_matches;

/**********************************************************************
 * Administration service attributes.
 **********************************************************************/

/* Administration service pluginstatus attribute */
/* This attribute is RESERVED for internal use by the aznAPI Admin Service */
AZN_EXTSPEC_CONST azn_string_t azn_admin_svc_pluginstatus;

/* Administration service protected object attribute */
AZN_EXTSPEC_CONST azn_string_t azn_admin_svc_pobj;

/* Administration service task attribute */
AZN_EXTSPEC_CONST azn_string_t azn_admin_svc_task;

/* Administration service results attribute */
AZN_EXTSPEC_CONST azn_string_t azn_admin_svc_results;

/**********************************************************************
 * aznAPI Function Prototypes
 **********************************************************************/


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_initialize(
  const azn_attrlist_h_t  init_data   /* in */,
  azn_attrlist_h_t        *init_info  /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_shutdown(
        );


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_init_set_code_set(
    const azn_string_t codeset        /* in */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_attrlist_add_entry(
    const azn_attrlist_h_t   attr_list      /* in */,
    const azn_string_t       attr_name      /* in */,
    const azn_string_t       string_value   /* in */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_attrlist_add_entry_buffer(
    const azn_attrlist_h_t   attr_list      /* in */,
    const azn_string_t       attr_name      /* in */,
    const azn_buffer_t       buffer_value   /* in */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_attrlist_add_entry_pobj(
    const azn_attrlist_h_t   attr_list     /* in */,
    const azn_string_t       attr_name     /* in */,
    const azn_pobj_t         pobj_value    /* in */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_attrlist_add_entry_ulong(
    const azn_attrlist_h_t   attr_list     /* in */,
    const azn_string_t       attr_name     /* in */,
    const azn_ulong_t        ulong_value   /* in */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_attrlist_create(
    azn_attrlist_h_t         *attr_list  /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_attrlist_delete(
    azn_attrlist_h_t         *attr_list  /* in, out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_attrlist_delete_entry(
    azn_attrlist_h_t         attr_list       /* in, out */,
    const azn_string_t       attr_name       /* in */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_attrlist_delete_entry_value(
    azn_attrlist_h_t         attr_list,      /* in, out */
    const azn_string_t       attr_name,      /* in */
    const unsigned int       value_index     /* in */
);

AZN_EXPORT
azn_attrlist_h_t
AZN_CALLTYPE
azn_attrlist_copy(
    const azn_attrlist_h_t   attr_list       /* in */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_attrlist_name_get_num(
    const azn_attrlist_h_t   attr_list       /* in */,
    const azn_string_t       attr_name       /* in */,
    unsigned int             *num_values     /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_attrlist_get_entry_buffer_value(
    const azn_attrlist_h_t   attr_list       /* in */,
    const azn_string_t       attr_name       /* in */,
    const unsigned int       value_index     /* in */,
    azn_buffer_t             *buffer_value   /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_attrlist_get_entry_type (
    const azn_attrlist_h_t   attr_list     /* in */,
    const azn_string_t       attr_name     /* in */,
    const unsigned int       value_index   /* in */,
    unsigned long            *value_type   /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_attrlist_get_entry_pobj_value(
    const azn_attrlist_h_t   attr_list      /* in */,
    const azn_string_t       attr_name      /* in */,
    const unsigned int       value_index    /* in */,
    azn_pobj_t               *pobj_value    /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_attrlist_get_entry_string_value(
    const azn_attrlist_h_t   attr_list      /* in */,
    const azn_string_t       attr_name      /* in */,
    const unsigned int       value_index    /* in */,
    azn_string_t             *string_value  /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_attrlist_get_entry_ulong_value(
    const azn_attrlist_h_t   attr_list     /* in */,
    const azn_string_t       attr_name     /* in */,
    const unsigned int       value_index   /* in */,
    azn_ulong_t              *ulong_value  /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_attrlist_get_names(
    const azn_attrlist_h_t   attr_list       /* in */,
    azn_string_t             *attr_names[]   /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_creds_create(
    azn_creds_h_t   *creds  /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_creds_delete(
    azn_creds_h_t   *creds  /* in, out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_id_get_creds(
    const azn_string_t    authzn_authority  /* in */,
    const azn_string_t    mechanism_id      /* in */,
    const azn_buffer_t    mechanism_info    /* in */,
    azn_creds_h_t         *authzn_creds     /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_pac_get_creds(
    const azn_buffer_t    pac           /* in */,
    const azn_string_t    pac_svc_id    /* in */,
    azn_creds_h_t         *new_creds    /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_creds_combine(
    const azn_creds_h_t   creds            /* in */,
    const azn_creds_h_t   creds_to_add     /* in */,
    azn_creds_h_t         *combined_creds  /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_creds_for_subject(
    const azn_creds_h_t   creds           /* in */,
    const unsigned int    subject_index   /* in */,
    azn_creds_h_t         *subject_creds  /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_creds_get_attrlist_for_subject(
    const azn_creds_h_t      creds            /* in */,
    const unsigned int       subject_index    /* in */,
    azn_attrlist_h_t         *creds_attrlist  /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_creds_get_attr_value_string(
    const azn_creds_h_t      creds           /* in */,
    const unsigned int       subject_index   /* in */,
    const azn_string_t       attr_name       /* in */,
    azn_string_t             *attr_value     /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_creds_set_attr_value_string(
    const azn_creds_h_t      creds           /* in */,
    const unsigned int       subject_index   /* in */,
    const azn_string_t       attr_name       /* in */,
    const azn_string_t       attr_value      /* in */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_creds_get_pac(
    const azn_creds_h_t   creds        /* in */,
    const azn_string_t    pac_svc_id   /* in */,
    azn_buffer_t          *pac         /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_creds_modify(
    const azn_creds_h_t      creds        /* in */,
    const azn_string_t       mod_svc_id   /* in */,
    const azn_attrlist_h_t   mod_info     /* in */,
    azn_creds_h_t            *new_creds   /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_creds_num_of_subjects(
    const azn_creds_h_t   creds              /* in */,
    unsigned int          *num_of_subjects   /* out */
);


AZN_EXPORT
azn_creds_h_t
AZN_CALLTYPE
azn_creds_copy(
    const azn_creds_h_t creds   /* in */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_creds_equal(
    const azn_creds_h_t cred1,      /* in */
    const azn_creds_h_t cred2,      /* in */
    azn_boolean_t       *is_equal   /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_decision_access_allowed(
    const azn_creds_h_t   creds                /* in */,
    const azn_string_t    protected_resource   /* in */,
    const azn_string_t    operation            /* in */,
    int                   *permission          /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_decision_access_allowed_ext(
    const azn_creds_h_t      creds                 /* in */,
    const azn_string_t       protected_resource    /* in */,
    const azn_string_t       operation             /* in */,
    const azn_attrlist_h_t   app_context           /* in */,
    int                      *permission           /* out */,
    azn_attrlist_h_t         *permission_info      /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_entitlement_get_entitlements(
    const azn_creds_h_t      creds                /* in */,
    const azn_string_t       entitlements_svc_id  /* in */,
    const azn_attrlist_h_t   app_context          /* in */,
    azn_attrlist_h_t         *entitlements        /* out */
);


AZN_EXPORT
unsigned int
AZN_CALLTYPE
azn_error_major(
    const azn_status_t   azn_status  /* in */
);


AZN_EXPORT
unsigned int
AZN_CALLTYPE
azn_error_minor(
    const azn_status_t   azn_status  /* in */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_error_get_string(
    const azn_status_t    aznstatus         /* in */,
    azn_string_t          *error_string     /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_error_minor_get_string(
    const unsigned int   minor_error            /* in */,
    azn_string_t         *minor_error_string    /* out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_release_buffer(
    azn_buffer_t   *buffer     /* in, out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_release_pobj(
    azn_pobj_t   *pobj         /* in, out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_release_string(
    azn_string_t   *string     /* in, out */
);


AZN_EXPORT
azn_status_t
AZN_CALLTYPE
azn_release_strings(
    azn_string_t   *strings[]  /* in, out */
);


#ifdef __cplusplus
}
#endif

#endif /* _OGAUTHZN_H */
