/*
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2001, 2003
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplicaion or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Copyright (C) Tivoli Systems/IBM.  2000
 *
 * NAME
 *     pdaudit - Policy Director Auditing Service header
 *
 * DESCRIPTION
 *     This header file contains the definitions of Auditing Service
 *     specific macros and the "C" API functions prototypes.
 */

#ifndef PD_AUDIT_H
#define PD_AUDIT_H

#include <stdio.h>
#include <stdarg.h>
#include <sys/types.h>
#include <amaudittypes.h>
#include <ogauthzn.h>
#include <pdlog.h>

#ifdef _WIN32
typedef long           pid_t;
typedef long           uid_t;
typedef long           gid_t;
#endif

typedef struct PDAuditCommon {
   char                 *linkage;
   PDAuditEnumResource  resource;
   char                 *originating_location;
   IVTstatus            fail_status;
   azn_creds_h_t        cred;
   char                 *protected_object;
   IVTbool              process_info_is_set;
   PDAuditEnumSystem    system;
   pid_t                pid;
   uid_t                uid;
   uid_t                eid;
   gid_t                gid;
   gid_t                egid;
   IVTbool              azn_info_is_set;
   IVTuint32            azn_permission;
   IVTuint32            azn_result;
   IVTstatus            azn_result_qualifier;
} PDAuditCommon;
/* DEPRECATED (end)                                                         */
/* ************************************************************************ */




/*
 * Log Service API interfaces.
 */

#ifdef __cplusplus
extern "C" {
#endif

    AZN_EXPORT void *PDAuditAttachSink1(char *,
                            IVTuint32,
                            PDAuditFilterFunction,
                            const char *encoding,   /* PD_AUDIT_ENCODING_UTF8 if data passed as UTF-8 */
                            IVTstatus *);

    AZN_EXPORT void *PDAuditAttachSink(char *,
                            IVTuint32,
                            PDAuditFilterFunction,
                            IVTstatus *);

    AZN_EXPORT void PDAuditDetachSink(void **,
                           IVTstatus *);

    AZN_EXPORT void *PDAuditEventStart(void *,
                            IVTuint32,                  /* Event id */
                            PDAuditEnumOutcome,
                            IVTstatus *);

    AZN_EXPORT void PDAuditEventAddText(void *event_h,
                             IVTstatus *status_p,
                             IVTuint32 hint,
                             const char *fmt, ...);

    /* use PDAuditEventSetCredInfo to override PDAuditCommon cred field. */
    AZN_EXPORT void PDAuditEventSetCredInfo(void *event_p,
                                                 azn_creds_h_t cred,
                                                 IVTstatus *status_p);


    /* use PDAuditEventSetNameValue to override any/all PDAuditCommon fields. */
    AZN_EXPORT void PDAuditEventSetNameValue(void *event_h,
                                                 const char *name,
                                                 const char *value,
                                                 IVTstatus *);

    /* use PDAuditEventSetNameVal to set name to numeric val - will get converted to string */
    AZN_EXPORT void PDAuditEventSetNameVal(void *event_h,
                                               const char *name,
                                               IVTuint32 val,
                                               IVTstatus *);


    /* PDAuditEventCommit maps to PDLogEventCommit */
    /* see pdlog.h for PDLogEventCommit signature  */
#define PDAuditEventCommit PDLogEventCommit

    /* DEPRECATED:                                                  */
    /* ----------                                                   */
    /* PDAuditEventAddCommon is no longer needed if all fields are  */
    /* described through PDAuditEventSetNameValue.                  */
    AZN_EXPORT void PDAuditEventAddCommon(void *event_h,
                             PDAuditCommon *c_p,
                             IVTstatus *);

#ifdef __cplusplus
}
#endif

#endif /* PD_AUDIT_H */
